BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ROLE_POLICY_SOD_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ROLE_POLICY_SOD_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ROLE_POLICY_SOD_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ROLE_POLICY_SOD_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ROLE_PERM_POLICY_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ROLE_PERM_POLICY_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ROLE_PERM_POLICY_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ROLE_PERM_POLICY_V';
  END LOOP;
END;
/

CREATE VIEW ROLE_POLICY_SOD_V AS
  SELECT DISTINCT
    policy.id AS policy_id,
    policy.name,
    policy.deleted,
    policy.description,
    policy.detection_status,
    policy.risk,
    policy.state,
    policy.unique_policy_id,
    policy.cost,
    policy.risk_mode,
    sod.name AS sod_name,
    sod.description AS sod_desc,
    sod.deleted AS sod_deleted,
    sod.state AS sod_state
  FROM role_policy policy
    LEFT JOIN sod_condition_item sod_cond_item ON sod_cond_item.unique_id = policy.unique_policy_id
    LEFT JOIN sod_condition sod_cond ON sod_cond.id = sod_cond_item.condition
    LEFT JOIN sod_policy sod ON sod.id = sod_cond.policy;

CREATE VIEW role_perm_policy_v AS
SELECT DISTINCT 
  policy.id,
  policy.name policy_name,
  policy.unique_policy_id,
  policy.description policy_description,
  policy.created_by,
  policy.changed_by,
  policy.creation_time,
  policy.update_time,
  policy.deleted,
  policy.last_detection_time,
  CASE
    WHEN policy.state = 'INACTIVE' AND policy.detection_status IS NULL OR policy.detection_status = 'INACTIVE' THEN 'Inactive'
    WHEN policy.state = 'ACTIVE' AND policy.detection_status = 'COMPLETED' OR policy.detection_status = 'RUNNING' THEN 'Active'
    WHEN policy.state = 'INACTIVE' AND policy.detection_status = 'INVALID' THEN 'Invalid from inactive'
    WHEN policy.state = 'INVALID' AND policy.detection_status = 'INVALID' THEN 'Invalid from active'
    ELSE NULL
  END current_state,
  policy.state actual_state,
  policy.detection_status,
  TO_CHAR(policy.risk_mode) policy_risk_mode,
  policy.risk policy_risk,
  policy.cost policy_cost,
  (SELECT DISTINCT (listagg(TO_CHAR(unique_owner_id),' | ')WITHIN GROUP (ORDER BY unique_owner_id)) user_owner_id
    FROM (SELECT DISTINCT powner.unique_owner_id 
      FROM policy_owners_v powner 
      LEFT OUTER JOIN role_policy rperm ON rperm.id = powner.role_policy  
      WHERE powner.is_user = 1 AND policy.id = rperm.id) subquery ) user_owner_id,
  (SELECT DISTINCT (listagg(TO_CHAR(unique_owner_id),' | ') WITHIN GROUP (ORDER BY unique_owner_id)) group_owner_id
    FROM (SELECT DISTINCT powner.unique_owner_id 
      FROM policy_owners_v powner 
      LEFT OUTER JOIN role_policy rperm ON rperm.id = powner.role_policy  
      WHERE powner.is_user = 0 AND policy.id = rperm.id) subquery2 ) group_owner_id,
  (SELECT DISTINCT (listagg(TO_CHAR(owner_name),' | ')WITHIN GROUP (ORDER BY owner_name)) user_owners
    FROM (SELECT DISTINCT powner.owner_name 
      FROM policy_owners_v powner 
      LEFT OUTER JOIN role_policy rperm ON rperm.id = powner.role_policy 
      WHERE powner.is_user = 1 AND policy.id = rperm.id) subquery ) user_owners,
  (SELECT DISTINCT (listagg(TO_CHAR(owner_name),' | ') WITHIN GROUP (ORDER BY owner_name)) group_owners
    FROM (SELECT DISTINCT powner.owner_name 
      FROM policy_owners_v powner 
      LEFT OUTER JOIN role_policy rperm ON rperm.id = powner.role_policy 
      WHERE powner.is_user = 0 AND policy.id = rperm.id) subquery2) group_owners,
  perm.unique_permission_id,
  perm.name perm_name,
  perm_user.unique_user_id permission_user_id,
  perm_user.display_name permission_user_name
FROM 
  role_policy policy
  LEFT JOIN policy_owner ON policy.id = policy_owner.role_policy
  LEFT JOIN role_permission role_perm ON policy.id = role_perm.policy
  LEFT JOIN spermission perm ON role_perm.unique_permission_id = perm.unique_permission_id AND perm.effective = 1
  LEFT JOIN suser_account_permission_role role_user ON role_user.role = policy.id
  LEFT JOIN suser perm_user ON perm_user.id = role_user.suser;

GRANT SELECT ON igops.ROLE_POLICY_SOD_V TO ig_report_role;
CREATE PUBLIC SYNONYM ROLE_POLICY_SOD_V FOR igops.ROLE_POLICY_SOD_V;

GRANT SELECT ON igops.role_perm_policy_v TO ig_report_role;
CREATE PUBLIC SYNONYM role_perm_policy_v FOR igops.role_perm_policy_v;